import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from server.logging_config import setup_logging


import logging

from django.template.loader import render_to_string
import time

import argparse


#########
## Shared functions
#########
def promptValidation(mandatory=False):
    if mandatory:
        args.yes = False
    if not args.yes:
        # Prompt the user for input
        response = input("Do you want to continue? (yes/no): ")
        # Check the user's response
        if response.lower() == 'yes' or response.lower() == 'y':
            # Continue the script
            print("Continuing the script...")
            # Add your code here to continue the script
            return True
        elif response.lower() == 'no' or response.lower() == 'n':
            # Stop the script
            print("Stopping the script...")
            # You might want to add cleanup code or exit the script gracefully here
            exit()
        else:
            # Invalid response
            print("Invalid response. Please enter 'yes' or 'no'.")
            exit()
            # You can choose to handle this case however you prefer
    else:
        print('--yes')

def displayBox(comandToExecute):
    os.system(comandToExecute)


#########
## Env setup
#########
from django.conf import settings
import os
from django import setup
# Make sure Django settings are configured
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'server.settings')
setup()
# needs import after django is setup
from django_q.tasks import async_task, result_group

#########
## Arg parsing
#########

# Parse command-line arguments
parser = argparse.ArgumentParser()
parser.add_argument("--template", help="Name of template to be used - root dir email_templates/. No need to attend .html in flag")
parser.add_argument("--subject", help="Subject used for the email")
parser.add_argument("--pj", help="Attach a file to the email - Enter the path of the file, ignoring the /united4earth/")

parser.add_argument("--recipient-file", help="Recipient file - must be single column csv, placed in /csv/ folder. No need to append .csv in flag")
parser.add_argument("--dev", action="store_true", help="Send to real recipient list")
parser.add_argument("--start-id",help="Start at a specific number in the list - here in case of an error/stop during sending")
parser.add_argument("--yes","-y",action="store_true",help="Yes to questions, exept critical")
parser.add_argument("--sleep",help="Customize time between emails send, default=5")
parser.add_argument("--fake",action="store_true",help="Do everything except actually sending the emails")
parser.add_argument("--log-level",help="define log level - Default django levels accepted + verbose")
parser.add_argument("--include-hostname",help="Include only specific email host")
parser.add_argument("--exclude-hostname",help="Include only specific email host")
args = parser.parse_args()

#### Setup logging
if args.log_level:
    #if we want different from server level
    logger = setup_logging(args.log_level)
else:
    logger = setup_logging()

#####
## Default figlet settings
#####
if args.dev:
    status = 'dev'
else:
    status = 'prod'

if status == 'prod':
    figlettext = 'Risefor Newsletter'
    # print(figlettext)
    os.system('figlet -l -k -w 80 -f doom -t '+figlettext+' | lolcat -a -s 200  -f')
    if args.fake:
        string = "Fake production run\nEverything will perform as prod, except sending"
    else:
        string = "WARNING\nYou will be sending real emails\nadd the --dev or --fake flag for testing\n"
        logging.basicConfig(
            filename='./newsletter-sender.log',  # Name of the log file
            level=logging.INFO,           # Log level
            format='%(asctime)s - %(levelname)s - %(message)s'  # Log format
        )
        logger.verbose("Production mode - logging activated")
    displayBox('echo "'+str(string)+'" | boxes -d c -a c -s 70x2')
    
    promptValidation(True)

else:
    figlettext = 'Newsletter Developing Mode'
    os.system('figlet -l -k -w 80 -f doom -t '+figlettext+' | lolcat -a -s 200  -f')
    
#########
## Set Email info
#########
if args.pj:
    logger.debug(f"There is a pj {args.pj}")

if args.sleep:
    sleepAfterSend = int(args.sleep)
else:
    sleepAfterSend = 5

# Email credentials
email_address = settings.DEFAULT_FROM_EMAIL # "community@risefor.org"

#####
## RECIPIENT HANDLING #####
#####
if status == 'dev':
    args.recipient_file = 'dev_email-list'

## Are we using a list or model
recipient_list = []
## Use File
string = "Recipient list section"
displayBox('echo "'+str(string)+'" | boxes -d c -a c -s 70x3')

if args.recipient_file:
    string = str("Using a file for recipient list :\n"+ args.recipient_file+".csv"+" in /csv/ folder")
    displayBox('echo "'+str(string)+'" | boxes -d caml -a c -s 70x4')

    promptValidation()
    emailListFile = './static/csv/'+args.recipient_file+'.csv'
    with open(emailListFile, encoding='utf-8') as emailList:
        count = 0
        for e in emailList:
            e = e.split(',')
            email = e[0].lower()
            source = e[1]
            userName = e[2]
            if '@' not in email:
                logger.error(f'not a valid email {email}')
            else:
                email = email.replace('\n','')
                email = email.replace(' ','')
                recipient_list.append({'email':email,'source':source,'userName':userName})
                count += 1
                logger.info(f"{count} {e}")
## Use Newsletter model
else:
    string = str("Using the 'Newletter List' of " + settings.INSTANCE_NAME + " as recipient list")
    displayBox('echo "'+str(string)+'" | boxes -d caml -a c -s 70x3')

    promptValidation()
    from risefor_lobbying.models import Newsletter
    newsletterList = Newsletter.objects.all()
    count = 0
    for e in newsletterList:
        ## noticed that some people write their physical adress in form
        if '@' not in e.userEmail:
            logger.error(f'not a valid email {e.userEmail}')
        else:
            count += 1
            recipient_list.append({'email':e.userEmail.lower(),'source':e.linkedActionId,'userName':e.fullname})
            logger.info(f"{count} {e.userEmail}")
        # print('all newletter',newsletterList.email)


# Your code to remove duplicates
unique_recipients = []
unique_emails = set()  # To keep track of unique email addresses
duplicates_removed = 0  # Initialize counter for duplicates removed

for recipient_info in recipient_list:
    email = recipient_info['email']
    remove_email = False
    # logger.debug(f"start email {email}")

    if (email not in unique_emails):
        if args.include_hostname or args.exclude_hostname:
            if args.include_hostname:
                hostnames = args.include_hostname.split(",")
            else:
                hostnames = args.exclude_hostname.split(",")

            email_domain = email.split('@')[-1] 
            logger.spam(f"include only {hostnames}")
            if args.include_hostname :
                if any(hostname in email_domain for hostname in hostnames):
                    unique_recipients.append(recipient_info)
                    unique_emails.add(email)
                    logger.debug(f"keep this email hostname {email} {hostnames}")
                else:
                    logger.debug(f"remove this email hostname {email} {hostnames}")
                    duplicates_removed += 1
            if args.exclude_hostname :
                if any(hostname in email_domain for hostname in hostnames):
                    logger.debug(f"remove this email hostname {email} {hostnames}")
                    duplicates_removed += 1
                else:
                    unique_recipients.append(recipient_info)
                    unique_emails.add(email)
                    logger.debug(f"keep this email hostname {email} {hostnames}")
        else:
            unique_recipients.append(recipient_info)
            unique_emails.add(email)

    else:
        duplicates_removed += 1

recipient_list = unique_recipients

logger.success(f"Duplicates removed: {duplicates_removed}")
logger.debug(f"final list {unique_recipients}")

### TO REVIEW
if args.start_id:
    print('\n\n####\nStart ID detected\n####\nDo you really Want to remove the first',int(args.start_id),"emails ?\n")
    promptValidation(True)
    rem = int(args.start_id)
    logger.info(f"removing : {recipient_list[0:rem]}")
    del recipient_list[0:rem]
    
    logger.verbose(f"list after del {recipient_list}")
    count = count - rem
string = str("There are "+str(count)+" people in the origin list.\nDuplicates removed: "+str(duplicates_removed)+"\nReal recipient list: " + str(len(recipient_list)))
displayBox('echo "'+str(string)+'" | boxes -d c -a c -s 70x4')

# os.system('echo "'+str(string)+'" | boxes -d ansi-rounded -a c -s 70x7')


# print("\n\nThere are",count,"people in the origin list.")
# print("Duplicates removed:", duplicates_removed)
# print("Real recipient list:", len(recipient_list))
print("You can review the list above if needed\n\nDoes that sound about right?")
promptValidation()

# Email template
# email_subject = "Interpelle le gouvernement Français pour mettre fin au génocide à Gaza"

if args.subject:
    email_subject = args.subject
else:
    email_subject = "Les actions Risefor | Participez aux interpellations en cours :)"
# email_subject = "Newsletter Action - Participez aux interpellations en cours ! | Risefor "

# print("\n\n\n######\nEmail Subject\n######\n\n")
string = "Email Subject"
displayBox('echo "'+str(string)+'" | boxes -d c -a c -s 70x3')

string = str("\nThe current email subject is :\n" + email_subject+"\n\n")
# displayBox('echo "'+string+'" | boxes -d caml -a c -s 70x3')
logger.info(string)

if args.yes == False:
    subjectResponse = input("Is this good for you? (yes/no): ")
    logger.verbose(f"response {subjectResponse}")
    if subjectResponse == "no" or subjectResponse == "n":
        email_subject = input("Input the subject you would like")
        logger.info(f"new subject : {email_subject}")
        promptValidation()

    # response = input("You inputed ",email_subject,". Is this good for you? (yes/no): ")
    # if response !== "yes" or !response == "y":
    else:
        logger.verbose("it's good")


if args.template:
    logger.verbose(f"Using the template : {args.template+'.html'}")
    template_email_html = get_template("email_templates/"+args.template+".html")
else:
    logger.warning(f"No template given, default to action recap email")
    template_email_html = get_template("email_templates/newsletter-ongoingActions.html")
    
string = str()
string = "###\nFinal Recap before sending\nMake sure everything is OK\n###\nYou are going to send to " + str(len(recipient_list))+" emails"
if args.recipient_file:
    # print('From',args.recipient_file+'.csv')
    string = string + '\n'+ str('From '+args.recipient_file+'.csv')
else:
    # print("From the Newletter Model of " + settings.INSTANCE_NAME)
    string = string + str('\nFrom the Newletter Model of '+ settings.INSTANCE_NAME)

if args.template:
    # print('Template : ',args.template+".html")
    string = string + str('\nTemplate : '+args.template+".html")

else:
    # print("Template : Ongoing actions")
    string = string + str("\nTemplate : Ongoing actions")

# displayBox('echo "'+str(string)+'" | boxes -d info -a c -s 70x7')

string = string + str('\nSubject: \n'+email_subject)
# string = str('\nSubject: \n'+email_subject)
string = string + str("\nSent from: "+email_address+"\nAs "+settings.INSTANCE_NAME)

displayBox('echo "'+str(string)+'" | boxes -d f90-box -a c -s 70x7')

promptValidation(mandatory=True)


emailcount = 0
for recipient_info in recipient_list:
    recipient_email = recipient_info['email']
    recipient_userName = recipient_info['userName']
    if recipient_info['source']:
        recipient_source = recipient_info['source'].replace('\n','')
        logger.debug(f"User {recipient_email} subscribed from {recipient_source}")
    else: 
        recipient_source = ''
        logger.debug(f"User {recipient_email} subscribed unkown {recipient_source}")
    emailcount += 1
     # Your logic to customize email content based on recipient source
    if 'ulule' in recipient_source.lower():
        logger.info(f"User is co fonder {recipient_userName}")
        cofounder = True
    else:
        cofounder = False
    d = {
        "emailSender": {
            "user_source": recipient_source,
            "userName": recipient_userName,
            "cofounder":cofounder,
        }
    }
    # print('context',d)
    email_html = template_email_html.render(d["emailSender"])
    email_text = email_html
    # print('context',email_html)

    # email = EmailMultiAlternatives( 
    #         subject=email_subject,
    #         # body=email_text, #fix here once we have a final email
    #         body=email_html,
    #         from_email='"{}" <{}>'.format(settings.INSTANCE_NAME, email_address),
    #         to=[recipient_email],
    #     )
    # email.attach_alternative(email_html, "text/html")
    # if args.pj:
    #     logger.notice(f"adding PJ {args.pj}")
    #     email.attach_file('./'+str(args.pj))
    
    email = {
        'template':[email_text,email_html],
        'subject':email_subject,
        # body=email_text, #fix here once we have a final email
        'from_email':[settings.INSTANCE_NAME, email_address],
        'to':[recipient_email],
    }

    if not args.fake:
        logger.success(f"sending email n {emailcount} to {recipient_email}")
        # email.send()
        async_task('server.tasks.send_multialternative_email',email,args,group="email_newsletter")
    else:
        logger.success(f"FAKE - sending email n {emailcount} to {recipient_email}")
    
    time.sleep(sleepAfterSend)



string = str(emailcount)+' emails have been sent'
# displayBox('echo "'+str(string)+'" | boxes -d bear -a c -s 70x7')
os.system('echo "'+str(string)+'" | boxes -d bear -a c -s 70x7 | lolcat -a -s 200  -f')
