import os
import environ

import coloredlogs
import logging
from verboselogs import VerboseLogger
from logging.config import dictConfig
import verboselogs

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
env = environ.Env(
    # Set casting, default value
    DEBUG=(bool, False)
)
env_file_path = os.path.join(BASE_DIR, '.env')
environ.Env.read_env(env_file_path)

DEBUG=env('DEBUG')

# Custom log level styles
coloredlogs.DEFAULT_LOG_LEVEL =env('LOG_LEVEL').strip()
coloredlogs.DEFAULT_FIELD_STYLES = {
    'asctime': {'color': 'green'},
    'hostname': {'color': 'magenta'},
    'levelname': {'bold': True, 'color': 'magenta'},
    # 'levelname': {'bold': True, },
    'name': {'color': 'blue'},
    'programname': {'color': 'cyan'},
    'thread': {'color': 'magenta'},
}
coloredlogs.DEFAULT_LEVEL_STYLES = {
    'critical': {'bold': True, 'color': 'red'},
    'error': {'color': 'red'},
    'success': {'bold': True,'color': 'green'},
    'warning': {'color': 'yellow'},
    'notice': {'color': 'blue','color': 'green'},
    'info': {'color': 'cyan'},
    'verbose': {'color': 'magenta'},
    'debug': {'color': 'magenta'},
    'spam': {'color': 'magenta'},
}


def setup_logging(log_level=None):
    LOG_LEVEL = log_level or env('LOG_LEVEL').strip()
    print('log level is set to',LOG_LEVEL)
    # Initialize the Verboselogs logger
    logger = VerboseLogger(__name__)
    
    # Define a custom Formatter with dynamic formatting based on log level
    class CustomFormatter(logging.Formatter):
        FORMATS = {
            'DEBUG': '%(asctime)s %(hostname)s %(name)s[%(process)d] %(levelname)s %(message)s',
            'INFO': '%(asctime)s %(levelname)s %(message)s',
            "NOTICE": '%(asctime)s %(levelname)s %(message)s',
            'WARNING': '%(asctime)s %(levelname)s %(message)s',
            'ERROR': '%(asctime)s %(levelname)s %(name)s %(message)s',
            'CRITICAL': '%(asctime)s %(hostname)s %(name)s[%(process)d] %(levelname)s %(message)s',
        }
    
    def get_log_format(log_level):
        return CustomFormatter.FORMATS.get(log_level, CustomFormatter.FORMATS['DEBUG'])

    # log_format = get_log_format(LOG_LEVEL)
    log_format = '%(levelname)s %(message)s'
    
    # Set up coloredlogs to colorize the output
    coloredlogs.install(level=LOG_LEVEL,fmt=log_format, logger=logger)
    
    return logger

